//
// C++ Interface: eel
//
// Description: 
//
//
// Author: Ryan Cunningham <rcunning@cs.ucla.edu>, (C) 2004
//
// Copyright: Ryan Cunningham
//
//

#ifndef EEL_H
#define EEL_H

#ifdef __cplusplus
extern "C" {
#endif

// Explicit Event Library (eel) Interface 0.8

// Some id types
struct eel_event;
struct eel_group;
typedef struct eel_group* const eel_group_id;
typedef struct eel_event* const eel_event_id;

// Basic types
enum eel_cb_return_value_enum {
    EEL_SUCCESS = 0,
    EEL_ERROR = -1
};
typedef enum eel_cb_return_value_enum eel_cb_return_value;
typedef eel_cb_return_value(*eel_callback)(eel_group_id gid, void *arg, int fd);

//
// API
//
// Library initialization / unintialization
int  eel_initialize(int exit_on_signal);
int  eel_uninitialize(void);
// Library
int  eel_dispatch_loop(void);
int  eel_dispatch_once(void);
int  eel_no_events(void);
void eel_exit_loop(void);
// Event functions, return NULL on failure
eel_event_id eel_add_timer(eel_group_id gid, eel_callback cb, void *cb_arg, int timeout_milliseconds);
eel_event_id eel_add_read(eel_group_id gid, eel_callback cb, void *cb_arg, int fd);
eel_event_id eel_add_write(eel_group_id gid, eel_callback cb, void *cb_arg, int fd);
eel_event_id eel_add_error(eel_group_id gid, eel_callback cb, void *cb_arg, int fd);
// Compound event functions, return NULL on failure
eel_event_id eel_add_read_timeout(
  eel_group_id gid, eel_callback cb, eel_callback timeout_cb, void *cb_arg, int fd, int timeout_milliseconds);
eel_event_id eel_add_write_timeout(
  eel_group_id gid, eel_callback cb, eel_callback timeout_cb, void *cb_arg, int fd, int timeout_milliseconds);
eel_event_id eel_add_error_timeout(
  eel_group_id gid, eel_callback cb, eel_callback timeout_cb, void *cb_arg, int fd, int timeout_milliseconds);
// Event operations
// Events must be in the same group to be exclusive, as well events must be valid
int  eel_make_exclusive(eel_event_id eid_in_out, eel_event_id eid_in);
// Group Operations
eel_group_id eel_new_group_id(void);
void eel_delete_group_id(eel_group_id gid);
int  eel_has_events(eel_group_id gid);
int  eel_remove_events(eel_group_id gid);

//
// some timer helpers for program conversion
//
#ifndef TIMEVAL_TO_MILLISECONDS
#define TIMEVAL_TO_MILLISECONDS(macro_arg_tv) \
  ( ((macro_arg_tv).tv_sec * 1000) + ((macro_arg_tv).tv_usec / 1000) )
#endif
#ifndef MILLISECONDS_TO_TIMEVAL
#define MILLISECONDS_TO_TIMEVAL(macro_arg_tv, macro_arg_milli) \
  do { (macro_arg_tv).tv_sec = ((macro_arg_milli)/1000); \
       (macro_arg_tv).tv_usec = (((macro_arg_milli)%1000)*1000); } while (0)
#endif

#ifdef __cplusplus
}
#endif

#endif // EEL_H
